@@ -5,26 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  Modifications by:
- #
- #    Brian P. Walenz beginning on 2015-NOV-27
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #    Sergey Koren beginning on 2015-DEC-02
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::Configure;
@@ -32,7 +20,10 @@ package canu::Configure;
 require Exporter;
 
 @ISA    = qw(Exporter);
-@EXPORT = qw(getAllowedResources displayMemoryValue displayGenomeSize configureAssembler);
+@EXPORT = qw(getAllowedResources
+             displayMemoryValue
+             displayGenomeSize
+             configureAssembler);
 
 use strict;
 use warnings "all";
@@ -40,6 +31,8 @@ no  warnings "uninitialized";
 use Carp qw(cluck);
 use Sys::Hostname;
 
+use List::Util qw(min max);
+
 use canu::Defaults;
 use canu::Execution;
 
@@ -163,19 +156,48 @@ sub expandRange ($$$$) {
 }
 
 
-sub findGridMaxMemoryAndThreads () {
-    my @grid   = split '\0', getGlobal("availableHosts");
-    my $maxmem = 0;
-    my $maxcpu = 0;
+sub findAbsoluteMax ($$) {
+    my $type   = shift @_;
+    my $class  = shift @_;
+
+    #  Find the local resources.
+
+    my $localmem  = getGlobal("localMemory");
+    my $localcpu  = getGlobal("localThreads");
+
+    #  Find the max memory available on any grid host.
+
+    my $gridmem = 0;
+    my $gridcpu = 0;
+    my @grid    = split '\0', getGlobal("availableHosts");
 
     foreach my $g (@grid) {
         my ($cpu, $mem, $num) = split '-', $g;
 
-        $maxmem = ($maxmem < $mem) ? $mem : $maxmem;
-        $maxcpu = ($maxcpu < $cpu) ? $cpu : $maxcpu;
+        $gridmem = ($gridmem < $mem) ? $mem : $gridmem;
+        $gridcpu = ($gridcpu < $cpu) ? $cpu : $gridcpu;
     }
 
-    return($maxmem, $maxcpu);
+    #  Threshold to any max limit.
+
+    my $maxmem  = getGlobal("maxMemory");
+    my $maxcpu  = getGlobal("maxThreads");
+
+    if (defined($maxmem)) {
+        $localmem = min($localmem, $maxmem);
+        $gridmem = min($gridmem, $maxmem);
+    }
+    if (defined($maxcpu)) {
+        $localcpu = min($localcpu, $maxcpu);
+        $gridcpu = min($gridcpu, $maxcpu);
+    }
+
+    #  Return one or the other or fail.
+
+    return(($class eq "grid") ? $gridmem : $localmem)   if ($type eq "memory");
+    return(($class eq "grid") ? $gridcpu : $localcpu)   if ($type eq "threads");
+
+    caExit("Invalid type '$type' supplied to findAbsoluteMax().", undef)
 }
 
 
@@ -190,7 +212,7 @@ sub getAllowedResources ($$$$$@) {
     my $alg  = shift @_;  #  Algorithm, e.g., "mhap", "ovl"
     my $err  = shift @_;  #  Report of things we can't run.
     my $all  = shift @_;  #  Report of things we can run.
-    my $uni  = shift @_;  #  There's only one task to run (bogart, gfa)
+    my $uni  = shift @_;  #  There's only one task to run (bogart)
     my $dbg  = shift @_;  #  Optional, report debugging stuff
 
     #  If no grid, or grid not enabled, everything falls under 'lcoal'.
@@ -203,42 +225,41 @@ sub getAllowedResources ($$$$$@) {
         caExit("invalid useGrid (" . getGlobal("useGrid") . ") and gridEngine (" . getGlobal("gridEngine") . "); found no execution hosts - is grid available from this host?", undef);
     }
 
-    #  Figure out limits.
+    #  Figure out absolute limits of job sizes.  These are just limits on
+    #  what values we'll iterate over when finding a job size.
 
     my $minMemory    = getGlobal("minMemory");
     my $minThreads   = getGlobal("minThreads");
 
-    my $maxMemory    = getGlobal("maxMemory");
-    my $maxThreads   = getGlobal("maxThreads");
+    my $maxMemory    = findAbsoluteMax("memory",  $class);  #  This is the min of any user-defined max and the
+    my $maxThreads   = findAbsoluteMax("threads", $class);  #  local/grid host maximum.  Not param maxMemory, etc.
+
+    caExit("maxMemory is not defined", undef)   if (!defined($maxMemory));
+    caExit("maxThreads is not defined", undef)  if (!defined($maxThreads));
 
     my $taskMemory   = getGlobal("${tag}${alg}Memory");   #  Algorithm limit, "utgovlMemory", etc.
     my $taskThreads  = getGlobal("${tag}${alg}Threads");  #
 
-    #  The task limits MUST be defined.
-
     caExit("${tag}${alg}Memory is not defined", undef)   if (!defined($taskMemory));
     caExit("${tag}${alg}Threads is not defined", undef)  if (!defined($taskThreads));
 
-    #  If the maximum limits aren't set, default to 'unlimited' (for the grid; we'll effectively filter
-    #  by the number of jobs we can fit on the hosts) or to the current hardware limits.
-
     if ($dbg) {
-        print STDERR "--\n";
-        print STDERR "-- ERROR\n";
-        print STDERR "-- ERROR  Limited to at least $minMemory GB memory via minMemory option\n"   if (defined($minMemory));
-        print STDERR "-- ERROR  Limited to at least $minThreads threads via minThreads option\n"   if (defined($minThreads));
-        print STDERR "-- ERROR  Limited to at most $maxMemory GB memory via maxMemory option\n"    if (defined($maxMemory));
-        print STDERR "-- ERROR  Limited to at most $maxThreads threads via maxThreads option\n"    if (defined($maxThreads));
+        my $minm  = getGlobal("minMemory");
+        my $mint  = getGlobal("minThreads");
+
+        my $maxm  = getGlobal("maxMemory");
+        my $maxt  = getGlobal("maxThreads");
+
+        undef $minm   if ($minm == 0);   #  The default is "0"; undef for easier testing below.
+        undef $mint   if ($mint == 0);
+
+        print STDERR "--\n"                              if (defined($minm) || defined($mint) || defined($maxm) || defined($maxt));
+        print STDERR "-- ERROR  minMemory=${minm}g\n"    if (defined($minm));
+        print STDERR "-- ERROR  maxMemory=${maxm}g\n"    if (defined($maxm));
+        print STDERR "-- ERROR  minThreads=${mint}\n"    if (defined($mint));
+        print STDERR "-- ERROR  maxThreads=${maxt}\n"    if (defined($maxt));
     }
 
-    #  Figure out the largest memory and threads that could ever be supported.  This lets us short-circuit
-    #  the loop below.
-
-    my ($gridMaxMem, $gridMaxThr) = findGridMaxMemoryAndThreads();
-
-    $maxMemory  = (($class eq "grid") ? $gridMaxMem : getPhysicalMemorySize())  if (!defined($maxMemory));
-    $maxThreads = (($class eq "grid") ? $gridMaxThr : getNumberOfCPUs())        if (!defined($maxThreads));
-
     #  Build a list of the available hardware configurations we can run on.  If grid, we get this
     #  from the list of previously discovered hosts.  If local, it's just this machine.
 
@@ -374,13 +395,13 @@ sub getAllowedResources ($$$$$@) {
         print STDERR "-- ERROR\n";
         print STDERR "-- ERROR  Task $tag$alg can't run on any available machines.\n";
         print STDERR "-- ERROR  It is requesting:\n";
-        print STDERR "-- ERROR    ${tag}${alg}Memory=", getGlobal("${tag}${alg}Memory"), " memory (gigabytes)\n";
+        print STDERR "-- ERROR    ${tag}${alg}Memory=", getGlobal("${tag}${alg}Memory"), " gigabytes\n";
         print STDERR "-- ERROR    ${tag}${alg}Threads=", getGlobal("${tag}${alg}Threads"), " threads\n";
         print STDERR "-- ERROR\n";
         print STDERR "-- ERROR  No available machine configuration can run this task.\n";
         print STDERR "-- ERROR\n";
         print STDERR "-- ERROR  Possible solutions:\n";
-        print STDERR "-- ERROR    Increase maxMemory\n"  if (defined(getGlobal("maxMemory")));
+        print STDERR "-- ERROR    Change maxMemory or maxThreads\n";
         print STDERR "-- ERROR    Change ${tag}${alg}Memory and/or ${tag}${alg}Threads\n";
         print STDERR "-- ERROR\n";
 
@@ -425,7 +446,7 @@ sub getAllowedResources ($$$$$@) {
         #  If already set (on the command line), reset if too big.
 
         if (getGlobal("${tag}${alg}Concurrency") > $nc) {
-            $err .= "-- Reset concurrency from " . getGlobal("${tag}${alg}Concurrency") . " to $nc.\n";
+            $err .= "-- Reset ${tag}${alg}Concurrency from " . getGlobal("${tag}${alg}Concurrency") . " to $nc.\n";
             setGlobal("${tag}${alg}Concurrency", $nc);
         }
 
@@ -463,26 +484,24 @@ sub getAllowedResources ($$$$$@) {
     elsif ($alg eq "red")       {  $nam = "(read error detection)"; }
     elsif ($alg eq "oea")       {  $nam = "(overlap error adjustment)"; }
     elsif ($alg eq "bat")       {  $nam = "(contig construction with bogart)"; }
-    elsif ($alg eq "dbg")       {  $nam = "(contig construction with wtdbg)"; }
     elsif ($alg eq "cns")       {  $nam = "(consensus)"; }
-    elsif ($alg eq "gfa")       {  $nam = "(GFA alignment and processing)"; }
     else {
         caFailure("unknown task '$alg' in getAllowedResources().", undef);
     }
 
-    my $mem  = substr("    $taskMemory",  -4) . " GB";
-    my $thr  = substr("    $taskThreads", -3) . " CPU" . (($taskThreads == 1) ? " " : "s");
-    my $job  = substr("    $concurrent",  -3) . " job" . (($concurrent == 1) ? " " : "s");
+    my $mem  = sprintf("%7.3f GB",  $taskMemory);
+    my $thr  = sprintf("%3d CPU%s", $taskThreads, (($taskThreads == 1) ? " " : "s"));
+    my $job  = sprintf("%3d job%s", $concurrent,  (($concurrent  == 1) ? " " : "s"));
 
-    my $memt = substr("     " . $taskMemory  * $concurrent, -4) . " GB";
-    my $thrt = substr("     " . $taskThreads * $concurrent, -4) . " CPU" . (($taskThreads * $concurrent == 1) ? " " : "s");
+    my $memt = sprintf("%8.3f GB",  $taskMemory  * $concurrent);
+    my $thrt = sprintf("%3d CPU%s", $taskThreads * $concurrent, (($taskThreads * $concurrent == 1) ? " " : "s"));
 
-    $mem  = " --- GB"     if ($taskMemory  == 0);
-    $thr  =  "--- CPUs"   if ($taskThreads == 0);
-    $job  =  "--- jobs"   if ($concurrent  == 0);
+    $mem  = "  -.--- GB"    if ($taskMemory  == 0);
+    $thr  = "  - CPUs"      if ($taskThreads == 0);
+    $job  = "  - jobs"      if ($concurrent  == 0);
 
-    $memt = " --- GB"     if ($taskMemory  * $concurrent == 0);
-    $thrt = " --- CPUs"   if ($taskThreads * $concurrent == 0);
+    $memt = "   -.--- GB"   if ($taskMemory  * $concurrent == 0);
+    $thrt = "  - CPUs"      if ($taskThreads * $concurrent == 0);
 
     my $t = substr("$tag$alg     ", 0, 7);
 
@@ -491,101 +510,32 @@ sub getAllowedResources ($$$$$@) {
         #$all .= "-- Memory and Threads configuration:\n"               if (!defined($concurrent));
 
         if (defined($concurrent)) {
-            $all .= "--                            (tag)Concurrency\n";
-            $all .= "--                     (tag)Threads          |\n";
-            $all .= "--            (tag)Memory         |          |\n";
-            $all .= "--        (tag)         |         |          |     total usage     algorithm\n";
-            $all .= "--        -------  ------  --------   --------  -----------------  -----------------------------\n";
+            $all .= "--                                (tag)Concurrency\n";
+            $all .= "--                         (tag)Threads          |\n";
+            $all .= "--                (tag)Memory         |          |\n";
+            $all .= "--        (tag)             |         |          |       total usage      algorithm\n";
+            $all .= "--        -------  ----------  --------   --------  --------------------  -----------------------------\n";
         } else {
-            $all .= "--                     (tag)Threads\n";
-            $all .= "--            (tag)Memory         |\n";
-            $all .= "--        (tag)         |         |  algorithm\n";
-            $all .= "--        -------  ------  --------  -----------------------------\n";
+            $all .= "--                         (tag)Threads\n";
+            $all .= "--                (tag)Memory         |\n";
+            $all .= "--        (tag)             |         |  algorithm\n";
+            $all .= "--        -------  ----------  --------  -----------------------------\n";
         }
     }
-    $all .= "-- Local: $t $mem  $thr x $job  $memt $thrt  $nam\n"      if ( defined($concurrent));
-    $all .= "-- Grid:  $t $mem  $thr  $nam\n"                          if (!defined($concurrent));
+    $all .= "-- Local: $t  $mem  $thr x $job  $memt $thrt  $nam\n"      if ( defined($concurrent));
+    $all .= "-- Grid:  $t  $mem  $thr  $nam\n"                          if (!defined($concurrent));
 
     return($err, $all);
 }
 
 
 
-
-#  Converts number with units to gigabytes.  If no units, gigabytes is assumed.
-sub adjustMemoryValue ($) {
-    my $val = shift @_;
-
-    return(undef)                     if (!defined($val));
-
-    return($1)                        if ($val =~ m/^(\d+\.{0,1}\d*)$/);
-    return($1 / 1024 / 1024)          if ($val =~ m/^(\d+\.{0,1}\d*)[kK]$/);
-    return($1 / 1024)                 if ($val =~ m/^(\d+\.{0,1}\d*)[mM]$/);
-    return($1)                        if ($val =~ m/^(\d+\.{0,1}\d*)[gG]$/);
-    return($1 * 1024)                 if ($val =~ m/^(\d+\.{0,1}\d*)[tT]$/);
-    return($1 * 1024 * 1024)          if ($val =~ m/^(\d+\.{0,1}\d*)[pP]$/);
-
-    die "Invalid memory value '$val'\n";
-}
-
-
-#  Converts gigabytes to number with units.
-sub displayMemoryValue ($) {
-    my $val = shift @_;
-
-    return(($val * 1024 * 1024)        . "k")   if ($val < adjustMemoryValue("1m"));
-    return(($val * 1024)               . "m")   if ($val < adjustMemoryValue("1g"));
-    return(($val)                      . "g")   if ($val < adjustMemoryValue("1t"));
-    return(($val / 1024)               . "t");
-}
-
-
-#  Converts number with units to bases.
-sub adjustGenomeSize ($) {
-    my $val = shift @_;
-
-    return(undef)               if (!defined($val));
-
-    return($1)                  if ($val =~ m/^(\d+\.{0,1}\d*)$/i);
-    return($1 * 1000)           if ($val =~ m/^(\d+\.{0,1}\d*)[kK]$/i);
-    return($1 * 1000000)        if ($val =~ m/^(\d+\.{0,1}\d*)[mM]$/i);
-    return($1 * 1000000000)     if ($val =~ m/^(\d+\.{0,1}\d*)[gG]$/i);
-    return($1 * 1000000000000)  if ($val =~ m/^(\d+\.{0,1}\d*)[tT]$/i);
-
-    die "Invalid genome size '$val'\n";
-}
-
-
-#  Converts bases to number with units.
-sub displayGenomeSize ($) {
-    my $val = shift @_;
-
-    return(($val))                        if ($val < adjustGenomeSize("1k"));
-    return(($val / 1000)          . "k")  if ($val < adjustGenomeSize("1m"));
-    return(($val / 1000000)       . "m")  if ($val < adjustGenomeSize("1g"));
-    return(($val / 1000000000)    . "g")  if ($val < adjustGenomeSize("1t"));
-    return(($val / 1000000000000) . "t");
-}
-
-
-
-
-
 #
 #  If minMemory or minThreads isn't defined, pick a reasonable pair based on genome size.
 #
 
 sub configureAssembler () {
 
-    #  Parse units on things the user possibly set.
-
-    setGlobal("genomeSize", adjustGenomeSize(getGlobal("genomeSize")));
-
-    setGlobal("executiveMemory", adjustMemoryValue(getGlobal("executiveMemory")));
-
-    setGlobal("minMemory",  adjustMemoryValue(getGlobal("minMemory")));
-    setGlobal("maxMemory",  adjustMemoryValue(getGlobal("maxMemory")));
-
     #  For overlapper and mhap, allow larger maximums for larger genomes.  More memory won't help
     #  smaller genomes, and the smaller minimums won't hurt larger genomes (which are probably being
     #  run on larger machines anyway, so the minimums won't be used).
@@ -730,23 +680,28 @@ sub configureAssembler () {
     #    Consensus memory is set based on tig size in Consensus.pm.
     #  Both are set to zero here, a special case that will configure only the thread component.
 
+    setGlobalIfUndef("cnsMemory",  "0");
+    setGlobalIfUndef("corMemory",  "0");
+
     if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
-        setGlobalIfUndef("cnsMemory",     "0");        setGlobalIfUndef("cnsThreads",      "1-4");
-        setGlobalIfUndef("corMemory",     "8");        setGlobalIfUndef("corThreads",      "4");
-        setGlobalIfUndef("cnsPartitions", "8");        setGlobalIfUndef("cnsPartitionMin", "15000");
-        setGlobalIfUndef("corPartitions", "64");       setGlobalIfUndef("corPartitionMin", "10000");
+        setGlobalIfUndef("corThreads", "4");
+        setGlobalIfUndef("cnsThreads", "1-4");
+    } else {
+        setGlobalIfUndef("corThreads", "4");
+        setGlobalIfUndef("cnsThreads", "2-8");
+    }
+
+    if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
+        setGlobalIfUndef("corPartitions",   "64");
+        setGlobalIfUndef("corPartitionMin", "10000");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("1g")) {
-        setGlobalIfUndef("cnsMemory",     "0");        setGlobalIfUndef("cnsThreads",      "2-8");
-        setGlobalIfUndef("corMemory",     "16");       setGlobalIfUndef("corThreads",      "4");
-        setGlobalIfUndef("cnsPartitions", "64");       setGlobalIfUndef("cnsPartitionMin", "20000");
-        setGlobalIfUndef("corPartitions", "128");      setGlobalIfUndef("corPartitionMin", "20000");
+        setGlobalIfUndef("corPartitions",   "128");
+        setGlobalIfUndef("corPartitionMin", "20000");
 
     } else {
-        setGlobalIfUndef("cnsMemory",     "0");        setGlobalIfUndef("cnsThreads",      "2-8");
-        setGlobalIfUndef("corMemory",     "24");       setGlobalIfUndef("corThreads",      "4");
-        setGlobalIfUndef("cnsPartitions", "256");      setGlobalIfUndef("cnsPartitionMin", "25000");
-        setGlobalIfUndef("corPartitions", "256");      setGlobalIfUndef("corPartitionMin", "40000");
+        setGlobalIfUndef("corPartitions",   "256");
+        setGlobalIfUndef("corPartitionMin", "40000");
     }
 
     #  Meryl too, basically just small or big.  This should really be using the number of bases
@@ -825,43 +780,38 @@ sub configureAssembler () {
         setGlobalIfUndef("oeaMemory", "8");           setGlobalIfUndef("oeaThreads", "1");
     }
 
-    #  And bogart and GFA alignment/processing.
-    #
-    #  GFA for genomes less than 40m is run in the canu process itself.
+    #  And bogart.
 
     if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
         setGlobalIfUndef("batMemory", "4-16");        setGlobalIfUndef("batThreads", "2-4");
-        setGlobalIfUndef("dbgMemory", "4-16");        setGlobalIfUndef("dbgThreads", "2-4");
-
-        setGlobalIfUndef("gfaMemory", "4-16");         setGlobalIfUndef("gfaThreads", "1-4");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("500m")) {
         setGlobalIfUndef("batMemory", "16-64");       setGlobalIfUndef("batThreads", "2-8");
-        setGlobalIfUndef("dbgMemory", "8-64");       setGlobalIfUndef("dbgThreads", "2-8");
-
-        setGlobalIfUndef("gfaMemory", "8-16");         setGlobalIfUndef("gfaThreads", "2-8");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("2g")) {
         setGlobalIfUndef("batMemory", "32-256");      setGlobalIfUndef("batThreads", "4-16");
-        setGlobalIfUndef("dbgMemory", "32-256");      setGlobalIfUndef("dbgThreads", "4-16");
-
-        setGlobalIfUndef("gfaMemory", "16-32");        setGlobalIfUndef("gfaThreads", "4-16");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("5g")) {
         setGlobalIfUndef("batMemory", "128-512");     setGlobalIfUndef("batThreads", "8-32");
-        setGlobalIfUndef("dbgMemory", "128-512");     setGlobalIfUndef("dbgThreads", "8-32");
-
-        setGlobalIfUndef("gfaMemory", "32-64");       setGlobalIfUndef("gfaThreads", "8-32");
 
     } else {
         setGlobalIfUndef("batMemory", "256-1024");    setGlobalIfUndef("batThreads", "16-64");
-        setGlobalIfUndef("dbgMemory", "128-1024");    setGlobalIfUndef("dbgThreads", "16-64");
-
-        setGlobalIfUndef("gfaMemory", "64-128");       setGlobalIfUndef("gfaThreads", "16-64");
     }
 
+    #  Log maxMemory setting.
 
+    {
+        my $maxcpu = getGlobal("maxThreads");            #  Get user-supplied limit.
+        my $maxmem = getGlobal("maxMemory");
 
+        if (($maxcpu > 0) ||
+            ($maxmem > 0)) {
+            printf STDERR "--\n";
+            printf STDERR "-- Job limits:\n";
+            printf(STDERR "--   %4d gigabytes memory  (maxMemory option).\n", $maxmem)    if (defined($maxmem));
+            printf(STDERR "--   %4d CPUs              (maxThreads option).\n", $maxcpu)   if (defined($maxcpu));
+        }
+    }
 
     #  Finally, use all that setup to pick actual values for each component.
     #
@@ -897,12 +847,9 @@ sub configureAssembler () {
     ($err, $all) = getAllowedResources("",    "oea",       $err, $all, 0);
 
     ($err, $all) = getAllowedResources("",    "bat",       $err, $all, 1)   if (uc(getGlobal("unitigger")) eq "BOGART");
-    ($err, $all) = getAllowedResources("",    "dbg",       $err, $all, 1)   if (uc(getGlobal("unitigger")) eq "WTDBG");
 
     ($err, $all) = getAllowedResources("",    "cns",       $err, $all, 0);
 
-    ($err, $all) = getAllowedResources("",    "gfa",       $err, $all, 1);
-
     #  Check some minimums.
 
     if ((getGlobal("ovsMemory") =~ m/^([0123456789.]+)-*[0123456789.]*$/) &&
