@@ -5,49 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  This file is derived from:
- #
- #    kmer/ESTmapper/scheduler.pm
- #    kmer/scripts/libBri.pm
- #    kmer/scripts/scheduler.pm
- #    src/pipelines/ca3g/Execution.pm
- #
- #  Modifications by:
- #
- #    Brian P. Walenz from 2003-JAN-03 to 2003-NOV-11
- #      are Copyright 2003 Applera Corporation, and
- #      are subject to the GNU General Public License version 2
- #
- #    Brian P. Walenz on 2004-MAR-22
- #      are Copyright 2004 Brian P. Walenz, and
- #      are subject to the GNU General Public License version 2
- #
- #    Brian P. Walenz from 2006-APR-07 to 2011-DEC-28
- #      are Copyright 2006,2008-2009,2011 J. Craig Venter Institute, and
- #      are subject to the GNU General Public License version 2
- #
- #    Brian P. Walenz from 2015-FEB-27 to 2015-SEP-11
- #      are Copyright 2015 Battelle National Biodefense Institute, and
- #      are subject to the BSD 3-Clause License
- #
- #    Brian P. Walenz beginning on 2015-NOV-03
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #    Sergey Koren beginning on 2015-NOV-25
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::Execution;
@@ -59,7 +24,6 @@ require Exporter;
              resetIteration
              touch
              makeExecutable
-             getInstallDirectory
              getJobIDShellCode
              getLimitShellCode
              getBinDirectory
@@ -301,12 +265,25 @@ sub stopAfter ($) {
 
     $stopAfter =~ tr/A-Z/a-z/;
 
-    if ((defined($stopAfter)) &&
-        (defined(getGlobal("stopAfter"))) &&
-        (getGlobal("stopAfter") eq $stopAfter)) {
-        print STDERR "Stop requested after '$stopAfter'.\n";
-        exit(0);
+    return   if (($stopAfter ne "theend") &&
+                 ($stopAfter ne getGlobal("stopAfter")));
+
+    if ($stopAfter ne "theend") {
+        print STDERR "--\n";
+        print STDERR "--  Stop requested after '$stopAfter'.\n";
     }
+
+    if (defined(getGlobal("onSuccess"))) {
+        print STDERR "--\n";
+        print STDERR "--  Running user-supplied termination command.\n";
+
+        runCommand(getGlobal("onExitDir"), getGlobal("onSuccess") . " " . getGlobal("onExitNam"));
+    }
+
+    print STDERR "--\n";
+    print STDERR "-- Bye.\n";
+
+    exit(0);
 }
 
 
@@ -330,15 +307,15 @@ sub resetIteration ($) {
 #
 #  To make it more trouble, shell scripts need to do all this by themselves.
 #
-sub getInstallDirectory () {
-    my $installDir = $FindBin::RealBin;
-
-    if ($installDir =~ m!^(.*)/\w+-\w+/bin$!) {
-        $installDir = $1;
-    }
-
-    return($installDir);
-}
+#sub getInstallDirectory () {
+#    my $installDir = ;
+#
+#    if ($installDir =~ m!^(.*)/\w+-\w+/bin$!) {
+#        $installDir = $1;
+#    }
+#
+#    return($installDir);
+#}
 
 
 #  Emits a block of shell code to parse the grid task id and offset.
@@ -372,6 +349,10 @@ sub getJobIDShellCode () {
     $string .= "  exit\n";
     $string .= "fi\n";
     $string .= "jobid=`expr -- \$baseid + \$offset`\n";
+    $string .= "if [ x\$baseid = x0 ]; then\n";
+    $string .= "  echo Error: jobid 0 is invalid\\; I need $taskenv set, or a job index on the command line.\n";
+    $string .= "  exit\n";
+    $string .= "fi\n";
     $string .= "if [ x\$$taskenv = x ]; then\n";
     $string .= "  echo Running job \$jobid based on command line options.\n";
     $string .= "else\n";
@@ -419,18 +400,14 @@ sub getLimitShellCode () {
 #  Used inside canu to find where binaries are located.
 #
 sub getBinDirectory () {
-    my $installDir = getInstallDirectory();
+    return($FindBin::RealBin);
 
-    my $syst = `uname -s`;                          chomp $syst;  #  OS implementation
-    my $arch = `uname -m | sed s/x86_64/amd64/`;    chomp $arch;  #  Hardware platform
-
-    my $path = "$installDir/$syst-$arch/bin";
-
-    if (! -d "$path") {
-        $path = $installDir;
-    }
-
-    return($path);
+    #my $idir = getInstallDirectory();
+    #my $path = $idir;
+    #
+    #$path = "$idir/bin"   if (-d "$idir/bin");
+    #
+    #return($path);
 }
 
 
@@ -438,7 +415,7 @@ sub getBinDirectory () {
 #  getBinDirectory.
 #
 sub getBinDirectoryShellCode () {
-    my $installDir = getInstallDirectory();
+    my $idir = $FindBin::RealBin;
     my $string;
 
     #  First, run any preExec command that might exist.
@@ -453,19 +430,12 @@ sub getBinDirectoryShellCode () {
     #  Then, setup and report paths.
 
     my $javaPath = getGlobal("java");
-    my $canu     = "\$bin/" . basename($0);
+    my $canu     = "\$bin/" . basename($0);   #  NOTE: $bin decided at script run time
 
     $string .= "\n";
     $string .= "#  Path to Canu.\n";
     $string .= "\n";
-    $string .= "syst=`uname -s`\n";
-    $string .= "arch=`uname -m | sed s/x86_64/amd64/`\n";
-    $string .= "\n";
-    $string .= "bin=\"$installDir/\$syst-\$arch/bin\"\n";
-    $string .= "\n";
-    $string .= "if [ ! -d \"\$bin\" ] ; then\n";
-    $string .= "  bin=\"$installDir\"\n";
-    $string .= "fi\n";
+    $string .= "bin=\"$idir\"\n";
     $string .= "\n";
     $string .= "#  Report paths.\n";
     $string .= "\n";
@@ -1265,6 +1235,20 @@ sub submitOrRunParallelJob ($$$$@) {
 
         purgeGridJobSubmitScripts($path, $script);
 
+        if (getGlobal("showNext")) {
+            print STDERR "--\n";
+            print STDERR "-- NEXT COMMANDS\n";
+            print STDERR "--\n";
+            print STDERR "\n";
+            print STDERR prettifyCommand("cd $path"), "\n";
+            foreach my $j (@jobs) {
+                my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
+
+                print STDERR prettifyCommand("./$cmd.sh") . "\n";
+            }
+            exit(0);
+        }
+
         foreach my $j (@jobs) {
             my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
 
@@ -1312,8 +1296,15 @@ sub submitOrRunParallelJob ($$$$@) {
                 }
 
                 if (uc(getGlobal("gridEngine")) eq "SLURM") {
+                    #  BPW has seen Slurm report "ERROR" instead of something
+                    #  useful here.  If that is seen, report the error to the
+                    #  screen and ignore this job.  We'll redo it on the next
+                    #  iteration (unless this is the second iteration, then
+                    #  we're screwed either way).
                     if (m/Submitted\sbatch\sjob\s(\d+)/) {
                         $jobName = $1;
+                    } elsif (m/ERROR/) {
+                        $jobName = undef;
                     } else {
                         $jobName = $_;
                     }
@@ -1325,13 +1316,17 @@ sub submitOrRunParallelJob ($$$$@) {
             }
             close(F);
 
-            if ($j =~ m/^\d+$/) {
+            if      (!defined($jobName)) {
+                print STDERR "-- '$cmd.sh' -> returned an error; job not submitted.\n";
+            } elsif ($j =~ m/^\d+$/) {
                 print STDERR "-- '$cmd.sh' -> job $jobName task $j.\n";
             } else {
                 print STDERR "-- '$cmd.sh' -> job $jobName tasks $j.\n";
             }
 
-            push @jobsSubmitted, $jobName;
+            if (defined($jobName)) {
+                push @jobsSubmitted, $jobName;
+            }
         }
 
         print STDERR "--\n";
@@ -1419,6 +1414,18 @@ sub submitOrRunParallelJob ($$$$@) {
             $st = $ed = $j;
         }
 
+        if (getGlobal("showNext")) {
+            print STDERR "--\n";
+            print STDERR "-- NEXT COMMANDS\n";
+            print STDERR "--\n";
+            print STDERR "\n";
+            print STDERR prettifyCommand("cd $path") . "\n";
+            for (my $i=$st; $i<=$ed; $i++) {
+                print STDERR prettifyCommand("./$script.sh $i") . "\n";
+            }
+            exit(0);
+        }
+
         for (my $i=$st; $i<=$ed; $i++) {
             schedulerSubmit("./$script.sh $i > ./" . buildOutputName($path, $script, $i) . " 2>&1");
         }
@@ -1516,7 +1523,11 @@ sub runCommand ($$) {
     #  If only showing the next command, show it and stop.
 
     if (getGlobal("showNext")) {
-        print STDERR "--NEXT-COMMAND\n";
+        print STDERR "--\n";
+        print STDERR "-- NEXT COMMAND\n";
+        print STDERR "--\n";
+        print STDERR "\n";
+        print STDERR prettifyCommand("cd $dir") . "\n";
         print STDERR "$dis\n";
         exit(0);
     }
