@@ -5,34 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  This file is derived from:
- #
- #    src/pipelines/ca3g/Meryl.pm
- #
- #  Modifications by:
- #
- #    Brian P. Walenz from 2015-FEB-27 to 2015-AUG-25
- #      are Copyright 2015 Battelle National Biodefense Institute, and
- #      are subject to the BSD 3-Clause License
- #
- #    Brian P. Walenz beginning on 2015-NOV-03
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #    Sergey Koren beginning on 2015-NOV-19
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::Meryl;
@@ -114,7 +94,9 @@ sub merylGenerateHistogram ($$) {
 
     return(undef)   if (! -e "$path/$name.histogram");
 
-    #  Load histogram data
+    #  Load histogram data, limited to a max count of 30,000, to prevent
+    #  these arrays from exhausting memory on very very frequent kmers.  It's
+    #  just for display anyway.
 
     my $numTotal    = 0;
     my $numDistinct = 0;
@@ -137,7 +119,7 @@ sub merylGenerateHistogram ($$) {
         $numDistinct = $1   if (m/distinct\s+(\d+)\s/);
         $numTotal    = $1   if (m/present\s+(\d+)\s/);
 
-        if ($v[0] =~ m/^\d+$/) {
+        if (($v[0] =~ m/^\d+$/) && ($v[0] < 30000)) {
             $maxCount             = $v[0];  #  maxCount is last count seen; histogram is sorted.
             $numDistinct[$v[0]]   = $v[1];
             $fractDistinct[$v[0]] = $v[2];
@@ -158,11 +140,43 @@ sub merylGenerateHistogram ($$) {
 
     my $TDmax  = 0;   #  Max count of of any block, excluding the first (we ignore the tail of this block when drawing the histogram)
 
+    my $maxRows = 40;   #  Number of rows for the count histogram.  We'll change the bin
+    my $rowIncr = 1;    #  size (rowIncr) based on the depth of coverage.
+
+    #  Find block sizes for the histogram.
+
     my $lo = 1;
     my $hi = 2;
     my $st = 1;
 
-    for (my $ii=0; $ii <= 40; $ii++) {
+    for (my $bb=1; $bb<6; $bb++) {
+        my $ft;
+
+        $lo = 1;
+        $hi = 2;
+        $st = 1;
+        for (my $ii=0; $ii <= $maxRows; $ii++) {
+            for (my $jj=$lo; $jj < $hi; $jj++) {
+                $ft = $fractTotal[$jj]   if (defined($fractTotal[$jj]))
+            }
+
+            $lo  = $hi;
+            $hi += $st;
+            $st += $bb;
+        }
+
+        if ($ft >= 0.95) {
+            $rowIncr = $bb;
+            last;
+        }
+    }
+
+    #  Find blocks for the histogram.
+
+    $lo = 1;
+    $hi = 2;
+    $st = 1;
+    for (my $ii=0; $ii <= $maxRows; $ii++) {
         for (my $jj=$lo; $jj < $hi; $jj++) {
             $TD[$ii] += $numDistinct[$jj];                                                   #  Sum the number of distinct mers we've seen
 
@@ -176,7 +190,7 @@ sub merylGenerateHistogram ($$) {
 
         $lo  = $hi;
         $hi += $st;
-        $st += 1;
+        $st += $rowIncr;
     }
 
     if ($TDmax == 0) {           #  A pathological case; if all kmers are unique,
@@ -186,7 +200,7 @@ sub merylGenerateHistogram ($$) {
     my $maxY   = 1;              #  Last count to include in the histogram.
     my $Xscale = $TDmax / 70;    #  Scale of each * in the histogram.
 
-    for (my $ii=0; $ii <= 40; $ii++) {
+    for (my $ii=0; $ii <= $maxRows; $ii++) {
         $maxY = $ii  if ($TD[$ii] > 0);
     }
 
@@ -199,8 +213,7 @@ sub merylGenerateHistogram ($$) {
     $lo = 1;
     $hi = 2;
     $st = 1;
-
-    for (my $ii=0; $ii<=40; $ii++) {
+    for (my $ii=0; $ii<=$maxRows; $ii++) {
         my $numXs = int($TD[$ii] / $Xscale);
 
         if ($numXs <= 70) {
@@ -219,7 +232,7 @@ sub merylGenerateHistogram ($$) {
 
         $lo  = $hi;
         $hi += $st;
-        $st += 1;
+        $st += $rowIncr;
     }
 
     $hist .= sprintf("--\n");
@@ -376,15 +389,15 @@ sub merylConfigure ($$) {
 
     my $mem = getGlobal("merylMemory");
     my $thr = getGlobal("merylThreads");
-    my $cov = getExpectedCoverage($tag, $asm);
+    my $cov = getExpectedCoverage($asm, $tag);
 
     open(F, "> $path/meryl-configure.sh");
     print F "#!" . getGlobal("shell") . "\n";
     print F "\n";
     print F getBinDirectoryShellCode();
-    print F "\n";
-    print F setWorkDirectoryShellCode($path);
-    print F fetchSeqStoreShellCode($asm, $path, "");
+    print F "\n";                                     #  setWDSC() should be OK here, but issue
+    #print F setWorkDirectoryShellCode($path);        #  #1740 had a problem with it, so it is
+    print F fetchSeqStoreShellCode($asm, $path, "");  #  disabled.  We're not sure why it broke.
 
     foreach my $ss (qw(01 02 04 06 08 12 16 20 24 32 40 48 56 64 96)) {
         next  if ($ss > $maxSplit);
@@ -626,7 +639,7 @@ sub merylConfigure ($$) {
     }
 
     if (getGlobal("${tag}Overlapper") eq "mhap") {
-        $mthresh   = int(5.0 * getExpectedCoverage($tag, $asm));
+        $mthresh   = int(5.0 * getExpectedCoverage($asm, $tag));
         $mdistinct = undef;
         $mwordfreq = getGlobal("${tag}MhapFilterThreshold");
     }
