@@ -5,34 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  This file is derived from:
- #
- #    src/pipelines/ca3g/OverlapBasedTrimming.pm
- #
- #  Modifications by:
- #
- #    Brian P. Walenz from 2015-MAR-16 to 2015-AUG-25
- #      are Copyright 2015 Battelle National Biodefense Institute, and
- #      are subject to the BSD 3-Clause License
- #
- #    Brian P. Walenz beginning on 2015-NOV-04
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #    Sergey Koren beginning on 2017-MAR-03
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::OverlapBasedTrimming;
@@ -229,22 +209,24 @@ sub loadTrimmedReads ($) {
 
     unlink("$path/$asm.loadTrimmedReads.err");
 
-    #  Save results.
+    #  Summarize and save results.
 
+    generateReadLengthHistogram("utg", $asm);
     stashSeqStore($asm);
 
-    #  Report reads.
+    #  If no trimmed reads were generated, stop now so we
+    #  preserve the stores.
 
-    addToReport("utgSeqStore", generateReadLengthHistogram("utg", $asm));
-
-    #stashFile("./$asm.trimmedReads.fasta.gz");
-
-    if (getGlobal("saveOverlaps") eq "0") {
+    if (getNumberOfReadsInStore($asm, "utg") > 0) {
+        print STDERR "--\n";
+        print STDERR "-- No trimmed reads generated, overlaps used for trimming saved.\n";
+    }
+    elsif (getGlobal("saveOverlaps") eq "0") {
         print STDERR "--\n";
         print STDERR "-- Purging overlaps used for trimming.\n";
-
         remove_tree("trimming/$asm.ovlStore")
-    } else {
+    }
+    else {
         print STDERR "--\n";
         print STDERR "-- Overlaps used for trimming saved.\n";
     }
@@ -264,63 +246,26 @@ sub dumpTrimmedReads ($) {
     my $cmd;
 
     goto allDone   if (fileExists("$asm.trimmedReads.fasta.gz"));
-    goto allDone   if (fileExists("$asm.trimmedReads.fastq.gz"));
     goto allDone   if (getGlobal("saveReads") == 0);
 
-    #  We need to skip this entire function if trimmed reads were not computed.
-    #  Otherwise, we incorrectly declare that all reads were trimmed out
-    #  and halt the assembly.
-    #
-    #  In cloud mode, we dump reads right after loading them, and to load them,
-    #  the 3-overlapbasedtrimming directory must exist, so we use that to decide if trimming
-    #  was attempted.
+    $cmd  = "$bin/sqStoreDumpFASTQ \\\n";
+    $cmd .= "  -trimmed \\\n";
+    $cmd .= "  -S ./$asm.seqStore \\\n";
+    $cmd .= "  -o ./$asm.trimmedReads.gz \\\n";
+    $cmd .= "  -fasta \\\n";
+    $cmd .= "  -trimmed -normal -nolibname \\\n";
+    $cmd .= "> ./$asm.trimmedReads.fasta.err 2>&1";
 
-    return         if (! -d "trimming/3-overlapbasedtrimming");
-
-    #  If no trimmed reads exist, don't bother trying to dump them.
-
-    if (getNumberOfReadsInStore($asm, "utg") > 0) {
-        $cmd  = "$bin/sqStoreDumpFASTQ \\\n";
-        $cmd .= "  -trimmed \\\n";
-        $cmd .= "  -S ./$asm.seqStore \\\n";
-        $cmd .= "  -o ./$asm.trimmedReads.gz \\\n";
-        $cmd .= "  -fasta \\\n";
-        $cmd .= "  -trimmed -normal -nolibname \\\n";
-        $cmd .= "> ./$asm.trimmedReads.fasta.err 2>&1";
-
-        if (runCommand(".", $cmd)) {
-            caExit("failed to output trimmed reads", "./$asm.trimmedReads.fasta.err");
-        }
-
-        unlink "./$asm.trimmedReads.fasta.err";
-
-        stashFile("$asm.trimmedReads.fasta.gz");
+    if (runCommand(".", $cmd)) {
+        caExit("failed to output trimmed reads", "./$asm.trimmedReads.fasta.err");
     }
 
-    #  If the trimmed reads file exists, report so.
-    #  Otherwise, report no trimmed reads, and generate fake outputs so we terminate.
+    unlink "./$asm.trimmedReads.fasta.err";
 
-    my $out;
+    stashFile("$asm.trimmedReads.fasta.gz");
 
-    $out = "$asm.trimmedReads.fasta.gz"   if (fileExists("$asm.trimmedReads.fasta.gz"));
-    $out = "$asm.trimmedReads.fastq.gz"   if (fileExists("$asm.trimmedReads.fastq.gz"));
-
-    if (defined($out)) {
-        print STDERR "--\n";
-        print STDERR "-- Trimmed reads saved in '$out'.\n";
-    } else {
-        print STDERR "--\n";
-        print STDERR "-- Yikes!  No trimmed reads generated!\n";
-        print STDERR "-- Can't proceed!\n";
-        print STDERR "--\n";
-        print STDERR "-- Generating empty outputs.\n";
-
-        runCommandSilently(".", "gzip -1vc < /dev/null > $asm.trimmedReads.gz   2> /dev/null", 0)   if (! -e "$asm.trimmedReads.gz");
-
-        stashFile("$asm.trimmedReads.fasta.gz");
-
-        generateOutputs($asm);
-    }
+    print STDERR "--\n";
+    print STDERR "-- Trimmed reads saved in '$asm.trimmedReads.fasta.gz'.\n";
 
   finishStage:
     generateReport($asm);
