@@ -5,34 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  This file is derived from:
- #
- #    src/pipelines/ca3g/OverlapMhap.pm
- #
- #  Modifications by:
- #
- #    Brian P. Walenz from 2015-MAR-27 to 2015-SEP-21
- #      are Copyright 2015 Battelle National Biodefense Institute, and
- #      are subject to the BSD 3-Clause License
- #
- #    Brian P. Walenz beginning on 2015-NOV-03
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #    Sergey Koren beginning on 2015-NOV-20
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::OverlapMhap;
@@ -108,7 +88,7 @@ sub mhapConfigure ($$$) {
     my ($numHashes, $minNumMatches, $threshold, $ordSketch, $ordSketchMer);
 
     if (!defined(getGlobal("${tag}MhapSensitivity"))) {
-        my $cov = getExpectedCoverage($tag, $asm);
+        my $cov = getExpectedCoverage($asm, $tag);
 
         setGlobal("${tag}MhapSensitivity", "low");                          #  Yup, super inefficient.  The code is
         setGlobal("${tag}MhapSensitivity", "normal")   if ($cov <  60);     #  compact and clear and runs once.
@@ -441,22 +421,14 @@ sub mhapConfigure ($$$) {
     print F "\n";
 
     my $stageDir = getGlobal("stageDirectory");
+    my $mhapPipe = getGlobal("${tag}mhapPipe");
 
-    if (defined($stageDir)) {
-        print F "#  Output staging enabled, write outputs in scratch space!\n";
-        print F "outPath=\"$stageDir\"\n";
-        print F "\n";
-    }
-
-    else {
-        print F "#  Output staging disabled, write all outputs in assembly directory.\n";
-        print F "outPath=\"./results\"\n";
-        print F "\n";
-    }
-
+    print F "outPath=\"$stageDir\"\n"          if ( defined($stageDir) && ($mhapPipe == 0));
+    print F "outPath=\"./results\"\n"          if (!defined($stageDir) && ($mhapPipe == 0));
+    print F "\n";
     print F "if [ ! -d ./results ]; then  mkdir -p ./results;  fi\n";
     print F "if [ ! -d ./blocks  ]; then  mkdir -p ./blocks;   fi\n";
-    print F "if [ ! -d \$outPath ]; then  mkdir -p \$outPath;  fi\n";
+    print F "if [ ! -d \$outPath ]; then  mkdir -p \$outPath;  fi\n"   if ($mhapPipe == 0);
     print F "\n";
 
     print F fetchFileShellCode("$path", "blocks/\$blk.dat", "");
@@ -466,12 +438,31 @@ sub mhapConfigure ($$$) {
     print F    fetchFileShellCode("$path", "blocks/\$ii", "  ");
     print F "done\n";
     print F "\n";
-
     print F "echo \"\"\n";
     print F "echo Running block \$blk in query \$qry\n";
     print F "echo \"\"\n";
     print F "\n";
-    print F "if [ ! -e \$outPath/\$qry.mhap ] ; then\n";
+    print F "if [ ! -e ./results/\$qry.mhap.ovb ] ; then\n";
+
+    if ($mhapPipe) {
+        print F "  #  Make a fifo so we can check return status on both\n";
+        print F "  #  mhap and mhapConvert, and still pipe results so we\n";
+        print F "  #  stop filling up disks.\n";
+        print F "  rm -f  \$qry-pipe\n";
+        print F "  mkfifo \$qry-pipe\n";
+        print F "\n";
+        print F "  #  Start up the consumer.\n";
+        print F "  \$bin/mhapConvert \\\n";
+        print F "    -S ../../$asm.seqStore \\\n";
+        print F "    -o ./results/\$qry.mhap.ovb.WORKING \\\n";
+        print F "    -minlength ", getGlobal("minOverlapLength"), " \\\n";
+        print F "    \$qry-pipe \\\n";
+        print F "  && \\\n";
+        print F "  touch ./results/\$qry.mcvt.success &\n";
+        print F "\n";
+    }
+
+    print F "  #  Start up the producer.\n";
     print F "  $javaPath $javaOpt -XX:ParallelGCThreads=",  getGlobal("${tag}mhapThreads"), " -server -Xms", $javaMemory, "m -Xmx", $javaMemory, "m \\\n";
     print F "    -jar $cygA \$bin/../share/java/classes/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
     print F "    --repeat-weight 0.9 --repeat-idf-scale 10 -k $merSize \\\n";
@@ -489,29 +480,51 @@ sub mhapConfigure ($$$) {
     print F " " . getGlobal("${tag}MhapOptions")         . " \\\n"   if (defined(getGlobal("${tag}MhapOptions")));
     print F "    -s $cygA ./blocks/\$blk.dat \$slf $cygB \\\n";
     print F "    -q $cygA queries/\$qry $cygB \\\n";
-    print F "  > \$outPath/\$qry.mhap.WORKING \\\n";
-    print F "  && \\\n";
-    print F "  mv -f \$outPath/\$qry.mhap.WORKING \$outPath/\$qry.mhap\n";
-    print F "fi\n";
-    print F "\n";
 
-    print F "if [   -e \$outPath/\$qry.mhap -a \\\n";
-    print F "     ! -e ./results/\$qry.ovb ] ; then\n";
-    print F "  \$bin/mhapConvert \\\n";
-    print F "    -S ../../$asm.seqStore \\\n";
-    print F "    -o ./results/\$qry.mhap.ovb.WORKING \\\n";
-    print F "    \$outPath/\$qry.mhap \\\n";
-    print F "  && \\\n";
-    print F "  mv ./results/\$qry.mhap.ovb.WORKING ./results/\$qry.mhap.ovb\n";
-    print F "fi\n";
-    print F "\n";
-
-    if (getGlobal('purgeOverlaps') ne "never") {
-        print F "if [   -e \$outPath/\$qry.mhap -a \\\n";
-        print F "       -e ./results/\$qry.mhap.ovb ] ; then\n";
-        print F "  rm -f \$outPath/\$qry.mhap\n";
+    if ($mhapPipe) {
+        print F "  > \$qry-pipe \\\n";
+        print F "  && \\\n";
+        print F "  touch ./results/\$qry.mhap.success\n";
+        print F "\n";
+        print F "  #  Wait for mhapConvert to finish.\n";
+        print F "  wait\n";
+        print F "\n";
+        print F "  #  Now that they're done, check status.\n";
+        print F "  if [ -e ./results/\$qry.mhap.success -a -e ./results/\$qry.mcvt.success ] ; then\n";
+        print F "    mv ./results/\$qry.mhap.ovb.WORKING ./results/\$qry.mhap.ovb\n";
+        print F "    rm -f ./results/\$qry.mhap.success\n";
+        print F "    rm -f ./results/\$qry.mcvt.success\n";
+        print F "  fi\n";
+        print F "\n";
+        print F "  #  And destroy the pipe.\n";
+        print F "  rm -f  \$qry-pipe\n";
         print F "fi\n";
         print F "\n";
+    } else {
+        print F "  > \$outPath/\$qry.mhap.WORKING \\\n";
+        print F "  && \\\n";
+        print F "  mv -f \$outPath/\$qry.mhap.WORKING \$outPath/\$qry.mhap\n";
+        print F "fi\n";
+        print F "\n";
+        print F "if [   -e \$outPath/\$qry.mhap -a \\\n";
+        print F "     ! -e ./results/\$qry.ovb ] ; then\n";
+        print F "  \$bin/mhapConvert \\\n";
+        print F "    -S ../../$asm.seqStore \\\n";
+        print F "    -o ./results/\$qry.mhap.ovb.WORKING \\\n";
+        print F "    -minlength ", getGlobal("minOverlapLength"), " \\\n";
+        print F "    \$outPath/\$qry.mhap \\\n";
+        print F "  && \\\n";
+        print F "  mv ./results/\$qry.mhap.ovb.WORKING ./results/\$qry.mhap.ovb\n";
+        print F "fi\n";
+        print F "\n";
+
+        if (getGlobal('purgeOverlaps') ne "never") {
+            print F "if [   -e \$outPath/\$qry.mhap -a \\\n";
+            print F "       -e ./results/\$qry.mhap.ovb ] ; then\n";
+            print F "  rm -f \$outPath/\$qry.mhap\n";
+            print F "fi\n";
+            print F "\n";
+        }
     }
 
     if (getGlobal("${tag}ReAlign") eq "1") {
