@@ -5,34 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  This file is derived from:
- #
- #    src/pipelines/ca3g/OverlapStore.pm
- #
- #  Modifications by:
- #
- #    Brian P. Walenz from 2015-FEB-27 to 2015-SEP-21
- #      are Copyright 2015 Battelle National Biodefense Institute, and
- #      are subject to the BSD 3-Clause License
- #
- #    Brian P. Walenz beginning on 2015-OCT-10
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #    Sergey Koren beginning on 2015-DEC-08
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::OverlapStore;
@@ -621,6 +601,66 @@ sub overlapStoreSorterCheck ($$$$$) {
 
 
 
+sub overlapStoreIndexerCheck ($$$$$) {
+    my $base       = shift @_;
+    my $asm        = shift @_;
+    my $tag        = shift @_;
+    my $numBuckets = shift @_;
+    my $numSlices  = shift @_;
+
+    my $bin   = getBinDirectory();
+    my $cmd;
+
+    #  If running with showNext, ovStoreIndexer will be run (manually
+    #  possibly) leaving an index file behind, but not renaming the store.
+    #  In that case, jump straight to finishStage and rename it.
+
+    goto allDone      if ((-d "$base/$asm.ovlStore") || (fileExists("$base/$asm.ovlStore.tar.gz")));
+
+    goto finishStage  if (-e  "$base/$asm.ovlStore.BUILDING/index");
+
+    #  Fetch the stats and index data.  If not using an object store, the
+    #  fetch does nothing, and since there is no file, the gzip/tar are
+    #  skipped.
+
+    for (my $ss=1; $ss<=$numSlices; $ss++) {
+        my $slice = substr("0000" . $ss, -4);
+
+        fetchFile("$base/$asm.ovlStore.BUILDING/$slice.statistics.tar.gz");
+
+        if (-e "$base/$asm.ovlStore.BUILDING/$slice.statistics.tar.gz") {
+            runCommandSilently("$base/$asm.ovlStore.BUILDING", "gzip -dc $slice.statistics.tar.gz | tar -xf -", 1);
+            unlink("$base/$asm.ovlStore.BUILDING/$slice.statistics.tar.gz");
+        }
+    }
+
+    $cmd  = "$bin/ovStoreIndexer \\\n";
+    $cmd .= "  -O  ./$asm.ovlStore.BUILDING \\\n";
+    $cmd .= "  -S ../$asm.seqStore \\\n";
+    $cmd .= "  -C  ./$asm.ovlStore.config \\\n";
+    $cmd .= "  -delete \\\n";
+    $cmd .= "> ./$asm.ovlStore.BUILDING.index.err 2>&1";
+
+    if (runCommand("$base", $cmd)) {
+        caExit("failed to build index for overlap store", "$base/$asm.ovlStore.BUILDING.index.err");
+    }
+
+    unlink "$base/$asm.ovlStore.BUILDING.index.err";
+
+  finishStage:
+    print STDERR "-- Overlap store indexer finished.\n";
+ 
+    rename "$base/$asm.ovlStore.BUILDING", "$base/$asm.ovlStore";
+
+    renameStashedFile("$base/$asm.ovlStore.BUILDING", "$base/$asm.ovlStore");
+    stashOvlStore($asm, $base);
+
+    #resetIteration("$tag-overlapStoreIndexerCheck");
+
+  allDone:
+}
+
+
 
 sub checkOverlapStore ($$) {
     my $base    = shift @_;
@@ -744,6 +784,64 @@ sub deleteOverlapIntermediateFiles ($$@) {
 
 
 
+#
+#  Scan the overlap store for old-format file names.  If any are found,
+#  create new-format names as symlinks to the original files.
+#
+sub updateOverlapStoreFiles ($$$) {
+    my $asm     = shift @_;
+    my $tag     = shift @_;
+    my $base    = shift @_;
+
+    if (-e "$base/$asm.ovlStore") {
+        my %oldToNew;
+        my @newExists;
+
+        #  Scan the store for any old-format files.
+        #  Remember if new-format links exist too.
+        open(L, "ls $base/$asm.ovlStore/ | ");
+        while (<L>) {
+            chomp;
+
+            if (m/^(\d\d\d\d)<(\d\d\d)>$/) {
+                my $old = "$1<$2>";
+                my $new = "$1-$2";
+
+                $oldToNew{$old} = $new;
+            }
+
+            if (m/^(\d\d\d\d)-(\d\d\d)$/) {
+                my $old = "$1<$2>";
+                my $new = "$1-$2";
+
+                push @newExists, $old;
+            }
+        }
+        close(L);
+
+        #  Forget about old-format names with new-format links already present.
+        foreach my $nn (@newExists) {
+            delete $oldToNew{$nn};
+        }
+
+        #  If there are any old-format names left, make links for them, logging
+        #  whatever we do.
+        if (scalar(%oldToNew) > 0) {
+            print STDERR "--\n";
+            print STDERR "-- UPDATING $base/$asm.ovlStore data files to new name format:\n";
+
+            foreach my $oo (keys %oldToNew) {
+                print STDERR "--   '$oo' -> '$oldToNew{$oo}'\n";
+                system("ln -s '$oo' '$base/$asm.ovlStore/$oldToNew{$oo}'");
+            }
+
+            print STDERR "--\n";
+        }
+    }
+}
+
+
+
 sub createOverlapStore ($$) {
     my $asm     = shift @_;
     my $tag     = shift @_;
@@ -756,6 +854,13 @@ sub createOverlapStore ($$) {
     $base = "trimming"    if ($tag eq "obt");
     $base = "unitigging"  if ($tag eq "utg");
 
+    #  A compatibility fix for renaming ovlStore data files from 0000<000> to 0000-000.
+    #  This change occurred around July 7 2020.
+
+    updateOverlapStoreFiles($asm, $tag, $base);
+
+    #  Now, if the directory exists, we have a store.
+
     goto allDone   if ((-d "$base/$asm.ovlStore") || (fileExists("$base/$asm.ovlStore.tar.gz")));
     goto allDone   if ((-d "$base/$asm.ctgStore") || (fileExists("$base/$asm.ctgStore.tar.gz")));
 
@@ -844,37 +949,7 @@ sub createOverlapStore ($$) {
         createOverlapStoreParallel ($base, $asm, $tag, $numBuckets, $numSlices, $sortMemory);
         overlapStoreBucketizerCheck($base, $asm, $tag, $numBuckets, $numSlices)   foreach (1..getGlobal("canuIterationMax") + 1);
         overlapStoreSorterCheck    ($base, $asm, $tag, $numBuckets, $numSlices)   foreach (1..getGlobal("canuIterationMax") + 1);
-
-        #  Fetch the stats and index data.  If not using an object store, the fetch does nothing,
-        #  and since there is no file, the gzip/tar are skipped.
-
-        for (my $ss=1; $ss<=$numSlices; $ss++) {
-            my $slice = substr("0000" . $ss, -4);
-
-            fetchFile("$base/$asm.ovlStore.BUILDING/$slice.statistics.tar.gz");
-
-            if (-e "$base/$asm.ovlStore.BUILDING/$slice.statistics.tar.gz") {
-                runCommandSilently("$base/$asm.ovlStore.BUILDING", "gzip -dc $slice.statistics.tar.gz | tar -xf -", 1);
-                unlink("$base/$asm.ovlStore.BUILDING/$slice.statistics.tar.gz");
-            }
-        }
-
-        $cmd  = "$bin/ovStoreIndexer \\\n";
-        $cmd .= "  -O  ./$asm.ovlStore.BUILDING \\\n";
-        $cmd .= "  -S ../$asm.seqStore \\\n";
-        $cmd .= "  -C  ./$asm.ovlStore.config \\\n";
-        $cmd .= "  -delete \\\n";
-        $cmd .= "> ./$asm.ovlStore.BUILDING.index.err 2>&1";
-
-        if (runCommand("$base", $cmd)) {
-            caExit("failed to build index for overlap store", "$base/$asm.ovlStore.BUILDING.index.err");
-        }
-
-        unlink "$base/$asm.ovlStore.BUILDING.index.err";
-        rename "$base/$asm.ovlStore.BUILDING", "$base/$asm.ovlStore";
-
-        renameStashedFile("$base/$asm.ovlStore.BUILDING", "$base/$asm.ovlStore");
-        stashOvlStore($asm, $base);
+        overlapStoreIndexerCheck   ($base, $asm, $tag, $numBuckets, $numSlices);
     }
 
   finishStage:
@@ -889,7 +964,7 @@ sub createOverlapStore ($$) {
 
     if ($tag eq "utg") {
         $cmd  = "$bin/ovStoreStats \\\n";
-        $cmd .= " -C " . getExpectedCoverage("utg", $asm). " \\\n";
+        $cmd .= " -C " . getExpectedCoverage($asm, "utg"). " \\\n";
         $cmd .= " -S ../$asm.seqStore \\\n";
         $cmd .= " -O  ./$asm.ovlStore \\\n";
         $cmd .= " -o  ./$asm.ovlStore \\\n";
