@@ -5,22 +5,14 @@
  #  sequencing reads into contigs.
  #
  #  This software is based on:
- #    'Celera Assembler' (http://wgs-assembler.sourceforge.net)
- #    the 'kmer package' (http://kmer.sourceforge.net)
- #  both originally distributed by Applera Corporation under the GNU General
- #  Public License, version 2.
+ #    'Celera Assembler' r4587 (http://wgs-assembler.sourceforge.net)
+ #    the 'kmer package' r1994 (http://kmer.sourceforge.net)
  #
- #  Canu branched from Celera Assembler at its revision 4587.
- #  Canu branched from the kmer project at its revision 1994.
+ #  Except as indicated otherwise, this is a 'United States Government Work',
+ #  and is released in the public domain.
  #
- #  Modifications by:
- #
- #    Brian P. Walenz beginning on 2018-APR-23
- #      are a 'United States Government Work', and
- #      are released in the public domain
- #
- #  File 'README.licenses' in the root directory of this distribution contains
- #  full conditions and disclaimers for each license.
+ #  File 'README.licenses' in the root directory of this distribution
+ #  contains full conditions and disclaimers.
  ##
 
 package canu::SequenceStore;
@@ -30,11 +22,11 @@ require Exporter;
 @ISA    = qw(Exporter);
 @EXPORT = qw(getNumberOfReadsInStore
              getNumberOfBasesInStore
+             getExpectedCoverage
              getSequenceStoreStats
              getSizeOfSequenceStore
-             getExpectedCoverage
-             generateReadLengthHistogram
-             checkSequenceStore);
+             createSequenceStore
+             generateReadLengthHistogram);
 
 use strict;
 use warnings "all";
@@ -54,23 +46,25 @@ use canu::Grid_Cloud;
 use Carp qw(longmess cluck confess);
 
 
+#  Return the number of reads in the store.
+#
+#  The 'all' category returns the number of reads that are in the store; essentailly, the maxID of any read.
+#  The 'cor' category returns the number of reads that are available for correction.
+#  The 'obt' category returns the number of reads that are available for trimming.
+#  The 'utg' category returns the number of reads that are available for assembly.
+#
 sub getNumberOfReadsInStore ($$) {
     my $asm    = shift @_;
     my $tag    = shift @_;
     my $nr     = 0;
 
-    confess  if (($tag ne "all") && ($tag ne "hap") && ($tag ne "cor") && ($tag ne "obt") && ($tag ne "utg"));
+    confess  if (($tag ne "all") &&
+                 ($tag ne "hap") &&
+                 ($tag ne "cor") &&
+                 ($tag ne "obt") &&
+                 ($tag ne "utg"));
 
-    #  No file, no reads.
-
-    return($nr)   if (! -e "./$asm.seqStore/info.txt");
-
-    #  Read the info file.  sqStoreCreate creates this at the end.
-    #
-    #  The 'all' category returns the number of reads that are in the store; essentailly, the maxID of any read.
-    #  The 'cor' category returns the number of reads that are available for correction.
-    #  The 'obt' category returns the number of reads that are available for trimming.
-    #  The 'utg' category returns the number of reads that are available for assembly.
+    return($nr)   if (! -e "./$asm.seqStore/info.txt");   #  No file, no reads.
 
     open(F, "< ./$asm.seqStore/info.txt") or caExit("can't open './$asm.seqStore/info.txt' for reading: $!", undef);
     while (<F>) {
@@ -84,33 +78,42 @@ sub getNumberOfReadsInStore ($$) {
     }
     close(F);
 
-    #print STDERR "-- Found $nr reads in '$asm.seqStore', for tag '$tag'\n";
-
     return($nr);
 }
 
 
 
-sub getNumberOfBasesInStore ($$) {
+#  Return the number of bases in the store.
+#
+#  Two modes of operation, depending on if the second option is supplied.
+#   - If $tag is "cor", "obt" or "utg", return the number of bases in reads
+#     that will be used for that stage.
+#   - If $tag is not supplied, return the maximum number of bases in
+#     any category ('raw', 'raw-trimmed', etc).
+#
+sub getNumberOfBasesInStore ($@) {
     my $asm    = shift @_;
     my $tag    = shift @_;
     my $nb     = 0;
 
-    confess  if (($tag ne "hap") && ($tag ne "cor") && ($tag ne "obt") && ($tag ne "utg"));
+    confess  if ((defined($tag)) &&
+                 ($tag ne "cor") &&
+                 ($tag ne "obt") &&
+                 ($tag ne "utg"));
 
-    #  No file, no bases.
-
-    return($nb)   if (! -e "./$asm.seqStore/info.txt");
-
-    #  Read the info file.  sqStoreCreate creates this at the end.
+    return($nb)   if (! -e "./$asm.seqStore/info.txt");   #  No file, no bases.
 
     open(F, "< ./$asm.seqStore/info.txt") or caExit("can't open './$asm.seqStore/info.txt' for reading: $!", undef);
     while (<F>) {
         if (m/^\s*(\d+)\s+(\d+)\s+(.*)\s*$/) {
-            $nb = $2 if (($3 eq "raw")               && ($tag eq "cor"));
-            $nb = $2 if (($3 eq "raw")               && ($tag eq "hap"));
-            $nb = $2 if (($3 eq "corrected")         && ($tag eq "obt"));
-            $nb = $2 if (($3 eq "corrected-trimmed") && ($tag eq "utg"));
+            $nb = $2   if (($3 eq "raw")               && ($tag eq "cor"));
+            $nb = $2   if (($3 eq "corrected")         && ($tag eq "obt"));
+            $nb = $2   if (($3 eq "corrected-trimmed") && ($tag eq "utg"));
+
+            $nb = $2   if (($3 eq "raw")               && ($nb < $2) && (!defined($tag)));
+            $nb = $2   if (($3 eq "raw-trimmed")       && ($nb < $2) && (!defined($tag)));
+            $nb = $2   if (($3 eq "corrected")         && ($nb < $2) && (!defined($tag)));
+            $nb = $2   if (($3 eq "corrected-trimmed") && ($nb < $2) && (!defined($tag)));
         }
     }
     close(F);
@@ -120,6 +123,25 @@ sub getNumberOfBasesInStore ($$) {
 
 
 
+#  Returns the estimated coverage in reads for the supplied tag (see
+#  getNumberOfBasesInStore for an important detail).
+#
+#  Returns a number rounded to hundredths of a genome, ###.##.
+#
+sub getExpectedCoverage ($@) {
+    my $asm    = shift @_;
+    my $tag    = shift @_;
+
+    confess  if ((defined($tag)) &&
+                 ($tag ne "cor") &&
+                 ($tag ne "obt") &&
+                 ($tag ne "utg"));
+
+    return(int(100 * getNumberOfBasesInStore($asm, $tag) / getGlobal("genomeSize")) / 100);
+}
+
+
+
 sub getSequenceStoreStats ($) {
     my $asm    = shift @_;
     my $bin    = getBinDirectory();
@@ -154,10 +176,10 @@ sub getSequenceStoreStats ($) {
 
         my @v = split '\s+', $_;
 
-        $numRaw++         if (($v[2] eq "raw")               && ($v[1] > 0));
-        $numRawTri++      if (($v[2] eq "raw-trimmed")       && ($v[1] > 0));
-        $numCor++         if (($v[2] eq "corrected")         && ($v[1] > 0));
-        $numCorTri++      if (($v[2] eq "corrected-trimmed") && ($v[1] > 0));
+        $numRaw    += $v[1]   if (($v[2] eq "raw")               && ($v[1] > 0));
+        $numRawTri += $v[1]   if (($v[2] eq "raw-trimmed")       && ($v[1] > 0));
+        $numCor    += $v[1]   if (($v[2] eq "corrected")         && ($v[1] > 0));
+        $numCorTri += $v[1]   if (($v[2] eq "corrected-trimmed") && ($v[1] > 0));
     }
     close(L);
 
@@ -198,21 +220,38 @@ sub getSizeOfSequenceStore ($) {
 
 
 
-sub getExpectedCoverage ($$) {
-    my $tag    = shift @_;
-    my $asm    = shift @_;
 
-    return(int(getNumberOfBasesInStore($asm, $tag) / getGlobal("genomeSize")));
+sub createSequenceStoreMetaDataFiles ($) {
+    my $asm = shift @_;
+    my $bin = getBinDirectory();
+
+    if (! -e "./$asm.seqStore/info.txt") {
+        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -stats > ./$asm.seqStore/info.txt 2> /dev/null", 1)) {
+            caExit("failed to generate $asm.seqStore/info.txt", undef);
+        }
+    }
+
+    if (! -e "./$asm.seqStore/libraries.txt") {
+        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -libs > ./$asm.seqStore/libraries.txt 2> /dev/null", 1)) {
+            caExit("failed to generate $asm.seqStore/libraries.txt", undef);
+        }
+    }
 }
 
 
-
-sub createSequenceStore ($$@) {
-    my $base   = shift @_;
+sub createSequenceStore ($@) {
     my $asm    = shift @_;
     my $bin    = getBinDirectory();
     my @inputs = @_;
 
+    #  If a seqStore exists, make sure that the metadata dumps exist, then
+    #  return silently.
+
+    if (-e "./$asm.seqStore") {
+        createSequenceStoreMetaDataFiles($asm);
+        return;
+    }
+
     #  Fail if there are no inputs.
 
     caExit("no input files specified, and store not already created, I have nothing to work on!", undef)
@@ -259,6 +298,10 @@ sub createSequenceStore ($$@) {
             print F "  -bias       " . getGlobal("readSamplingBias") . " \\\n";
         }
 
+        if (getGlobal("homoPolyCompress") > 0) {   #  Tell sqStoreCreate that these reads should
+            print F "  -homopolycompress \\\n";    #  be homopoly compressed, and so length filtering
+        }                                          #  should be based on that length.
+
         foreach my $iii (@inputs) {
             if ($iii =~ m/^(.*)\0(.*)$/) {
                 my $tech = $1;
@@ -296,23 +339,72 @@ sub createSequenceStore ($$@) {
         caExit("sqStoreCreate failed; boom!", "./$asm.seqStore.err");
     }
 
-    if (! -e "./$asm.seqStore/info.txt") {
-        caExit("sqStoreCreate failed; no info file", "./$asm.seqStore.err");
+    if (! -e "./$asm.seqStore/info") {
+        caExit("sqStoreCreate failed; no 'info' file", "./$asm.seqStore.err");
     }
+
+    createSequenceStoreMetaDataFiles($asm);
+
+    #  Set or unset the homopolymer compression flag.  This is also initially
+    #  created by sqStoreCreate (also based on the homoPolyCompress flag) but
+    #  we allow canu to explicitly change later.  Though doing that isn't
+    #  recommended.
+
+    if (defined(getGlobal("homoPolyCompress"))) {
+        touch("./$asm.seqStore/homopolymerCompression");
+    } else {
+        unlink("./$asm.seqStore/homopolymerCompression");
+    }
+
+    #  We don't really know what read type was loaded, and so we attempt to
+    #  generate all read length histograms.  If there is no data, no
+    #  histogram will be made.
+
+    generateReadLengthHistogram("cor", $asm);
+    generateReadLengthHistogram("obt", $asm);
+    generateReadLengthHistogram("utg", $asm);
+
+    #  Check if coverage is too low.  We don't actually know (easily) what
+    #  type of reads were just loaded, and so we check the maximum found
+    #  in the store.
+
+    my $curcov = getExpectedCoverage($asm);
+    my $mincov = getGlobal("minInputCoverage");
+
+    if ($curcov < $mincov) {
+        print STDERR "--\n";
+        print STDERR "-- ERROR:  Read coverage ($curcov) lower than allowed.\n";
+        print STDERR "-- ERROR:    minInputCoverage  = $mincov\n";
+        print STDERR "-- ERROR:\n";
+        print STDERR "-- ERROR:  This could be caused by an incorrect genomeSize.\n";
+        print STDERR "-- ERROR:\n";
+        print STDERR "-- ERROR:  You can force Canu to continue by decreasing parameter\n";
+        print STDERR "-- ERROR:  minInputCoverage.  Be warned that the quality of corrected\n";
+        print STDERR "-- ERROR:  reads and/or contiguity of contigs will be poor.\n";
+        print STDERR "--\n";
+
+        caExit("", undef);
+    }
+
+    #  Stop if requested.
+
+    stopAfter("sequenceStore");
 }
 
 
 
 sub generateReadLengthHistogram ($$) {
-    my $tag    = shift @_;
-    my $asm    = shift @_;
-    my $bin    = getBinDirectory();
+    my $tag      = shift @_;
+    my $asm      = shift @_;
+    my $bin      = getBinDirectory();
 
     my $reads    = getNumberOfReadsInStore($asm, $tag);
     my $bases    = getNumberOfBasesInStore($asm, $tag);
-    my $coverage = int(100 * $bases / getGlobal("genomeSize")) / 100;
+    my $coverage = getExpectedCoverage($asm, $tag);
     my $hist;
 
+    return   if ($reads == 0);
+
     my @rl;
     my @hi;
 
@@ -326,12 +418,12 @@ sub generateReadLengthHistogram ($$) {
         $cmd .= "  -raw \\\n"                  if ($tag eq "cor");
         $cmd .= "  -corrected \\\n"            if ($tag eq "obt");
         $cmd .= "  -corrected -trimmed \\\n"   if ($tag eq "utg");
-        $cmd .= "  -histogram \\\n";
+        $cmd .= "  -uncompressed \\\n";
         $cmd .= "  -lengths \\\n";
         $cmd .= "> ./$asm.seqStore/readlengths-$tag.dat \\\n";
         $cmd .= "2> ./$asm.seqStore/readlengths-$tag.err \n";
 
-        if (runCommand(".", $cmd) > 0) {
+        if (runCommandSilently(".", $cmd, 1)) {
             caExit("sqStoreDumpMetaData failed", "./$asm.seqStore/readlengths-$tag.err");
         }
 
@@ -340,27 +432,23 @@ sub generateReadLengthHistogram ($$) {
         my $gnuplot = getGlobal("gnuplot");
         my $format  = getGlobal("gnuplotImageFormat");
 
-        if ($gnuplot) {
-            open(F, "> ./$asm.seqStore/readlengths-$tag.gp") or caExit("can't open './$asm.seqStore/readlengths-$tag.gp' for writing: $!", undef);
-            print F "set title 'read length'\n";
-            print F "set xlabel 'read length, bin width = 250'\n";
-            print F "set ylabel 'number of reads'\n";
-            print F "\n";
-            print F "binwidth=250\n";
-            print F "set boxwidth binwidth\n";
-            print F "bin(x,width) = width*floor(x/width) + binwidth/2.0\n";
-            print F "\n";
-            print F "set terminal $format size 1024,1024\n";
-            print F "set output './$asm.seqStore/readlengths-$tag.$format'\n";
-            print F "plot [] './$asm.seqStore/readlengths-$tag.dat' using (bin(\$1,binwidth)):(1.0) smooth freq with boxes title ''\n";
-            close(F);
+        open(F, "> ./$asm.seqStore/readlengths-$tag.gp") or caExit("can't open './$asm.seqStore/readlengths-$tag.gp' for writing: $!", undef);
+        print F "set title 'read length'\n";
+        print F "set xlabel 'read length, bin width = 250'\n";
+        print F "set ylabel 'number of reads'\n";
+        print F "\n";
+        print F "binwidth=250\n";
+        print F "set boxwidth binwidth\n";
+        print F "bin(x,width) = width*floor(x/width) + binwidth/2.0\n";
+        print F "\n";
+        print F "set terminal $format size 1024,1024\n";
+        print F "set output './$asm.seqStore/readlengths-$tag.$format'\n";
+        print F "plot [] './$asm.seqStore/readlengths-$tag.dat' using (bin(\$1,binwidth)):(1.0) smooth freq with boxes title ''\n";
+        close(F);
 
-            if (runCommandSilently(".", "$gnuplot < /dev/null ./$asm.seqStore/readlengths-$tag.gp > /dev/null 2>&1", 0)) {
-                print STDERR "--\n";
-                print STDERR "-- WARNING: gnuplot failed.\n";
-                print STDERR "--\n";
-                print STDERR "----------------------------------------\n";
-            }
+        #  Run gnuplot, but don't care if it blows up.
+        if ($gnuplot) {
+            runCommandSilently(".", "$gnuplot < /dev/null ./$asm.seqStore/readlengths-$tag.gp > /dev/null 2>&1", 0);
         }
     }
 
@@ -374,11 +462,12 @@ sub generateReadLengthHistogram ($$) {
         $cmd .= "  -raw \\\n"                  if ($tag eq "cor");
         $cmd .= "  -corrected \\\n"            if ($tag eq "obt");
         $cmd .= "  -corrected -trimmed \\\n"   if ($tag eq "utg");
+        $cmd .= "  -uncompressed \\\n";
         $cmd .= "  -histogram \\\n";
         $cmd .= "> ./$asm.seqStore/readlengths-$tag.txt \\\n";
         $cmd .= "2> ./$asm.seqStore/readlengths-$tag.err \n";
 
-        if (runCommand(".", $cmd) > 0) {
+        if (runCommandSilently(".", $cmd, 1)) {
             caExit("sqStoreDumpMetaData failed", "./$asm.seqStore/readlengths-$tag.err");
         }
 
@@ -393,9 +482,6 @@ sub generateReadLengthHistogram ($$) {
     $hist .= "--   Found $bases bases ($coverage times coverage).\n";
 
     if (-e "./$asm.seqStore/readlengths-$tag.txt") {
-        #$hist .= "--\n";
-        #$hist .= "--   Read length histogram (one '*' equals " . int(100 * $scale) / 100 . " reads):\n";
-
         open(F, "< ./$asm.seqStore/readlengths-$tag.txt") or caExit("can't open './$asm.seqStore/readlengths-$tag.txt' for reading: $!", undef);
         while (<F>) {
             $hist .= "--    $_";
@@ -405,146 +491,23 @@ sub generateReadLengthHistogram ($$) {
 
     #  Return the ASCII histogram.
 
-    return($hist);
-}
-
-
-
-sub checkSequenceStore ($$@) {
-    my $base;
-    my $asm    = shift @_;
-    my $tag    = shift @_;
-    my $bin    = getBinDirectory();
-    my @inputs = @_;
-
-    $base = "haplotype"   if ($tag eq "hap");
-    $base = "correction"  if ($tag eq "cor");
-    $base = "trimming"    if ($tag eq "obt");
-    $base = "unitigging"  if ($tag eq "utg");
-
-    #  Try fetching the store from object storage.  This might not be needed in all cases (e.g.,
-    #  between mhap precompute and mhap compute), but it greatly simplifies stuff, like immediately
-    #  here needing to check if the store exists.
-
-    fetchSeqStore($asm);
-
-    #  We cannot abort this step anymore.  If trimming is skipped, we need ro promote the corrected
-    #  reads to trimmed reads, and also re-stash the store.
-    #
-    #goto allDone    if (getNumberOfReadsInStore($asm, $tag) > 0);
-
-    #  Create the store.
-
-    my $storeCreated = 0;
-    my $histAndStash = 0;
-
-    if (! -e "./$asm.seqStore") {
-        createSequenceStore($base, $asm, @inputs);
-
-        $storeCreated = 1;
-        $histAndStash = 1;
-    }
-
-    #  Dump the list of libraries.  Various parts use this for various stuff.
-
-    if (! -e "./$asm.seqStore/info.txt") {
-        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -stats > ./$asm.seqStore/info.txt 2> /dev/null", 1)) {
-            caExit("failed to generate $asm.seqStore/info.txt", undef);
-        }
-    }
-
-    if (! -e "./$asm.seqStore/libraries.txt") {
-        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -libs > ./$asm.seqStore/libraries.txt 2> /dev/null", 1)) {
-            caExit("failed to generate $asm.seqStore/libraries.txt", undef);
-        }
-    }
-
-    #  Query how many reads we have.
-
-    my $nCor = getNumberOfReadsInStore($asm, "cor");   #  Number of corrected reads ready for OBT.
-    my $nOBT = getNumberOfReadsInStore($asm, "obt");   #  Number of corrected reads ready for OBT.
-    my $nAsm = getNumberOfReadsInStore($asm, "utg");   #  Number of trimmed reads ready for assembly.
-
-    #  Refuse to assemble uncorrected reads.
-
-    if (($tag eq "utg") &&    #  Assembling.
-        ($nCor >  0) &&       #  And raw reads exist.
-        ($nOBT == 0) &&       #  But no corrected reads.
-        ($nAsm == 0)) {       #  And no trimmed reads!
-
-        print STDERR "-- Unable to assemble uncorrected reads.\n";
-
-        caExit("unable to assemble uncorrected reads", undef);
-    }
-
-    #  Set or unset the homopolymer compression flag.
-
-    if (defined(getGlobal("homoPolyCompress"))) {
-        touch("./$asm.seqStore/homopolymerCompression");
-    } else {
-        unlink("./$asm.seqStore/homopolymerCompression");
-    }
-
-    #  Make a histogram and stash the (updated) store.
-
-    if ($histAndStash) {
-        addToReport("${tag}SeqStore", generateReadLengthHistogram($tag, $asm));
-        stashSeqStore($asm);
-    }
-
-    #  Refuse to conitnue if there are no reads.
-
-    if ((($tag eq "cor") && ($nCor == 0)) ||
-        (($tag eq "obt") && ($nOBT == 0)) ||
-        (($tag eq "utg") && ($nAsm == 0))) {
-
-        print STDERR "--\n";              #  Nothing clever here, just keeping text aligned.
-        print STDERR "-- WARNING:\n";
-        print STDERR "-- WARNING:  No raw ",       "reads detected.  Cannot proceed; empty outputs generated.\n"  if ($tag eq "cor");
-        print STDERR "-- WARNING:  No corrected ", "reads detected.  Cannot proceed; empty outputs generated.\n"  if ($tag eq "obt");
-        print STDERR "-- WARNING:  No trimmed ",   "reads detected.  Cannot proceed; empty outputs generated.\n"  if ($tag eq "utg");
-        print STDERR "-- WARNING:\n";
-        print STDERR "--\n";
-
-        generateOutputs($asm);
-
-        return(0);
-    }
-
-    #  Stop if there isn't enough coverage.
-
-    my $minInputCov = getGlobal("minInputCoverage");
-    my $minStoreCov = getGlobal("stopOnLowCoverage");
-
-    my $coverage = int(100 * getNumberOfBasesInStore($asm, $tag) / getGlobal("genomeSize")) / 100;
-
-    if ((($coverage < $minInputCov) && ($storeCreated)) ||
-        (($coverage < $minStoreCov))) {
-        print STDERR "--\n";
-        print STDERR "-- ERROR:  Read coverage ($coverage) lower than allowed.\n";
-        print STDERR "-- ERROR:    minInputCoverage  = $minInputCov\n";
-        print STDERR "-- ERROR:    stopOnLowCoverage = $minStoreCov\n";
-        print STDERR "-- ERROR:\n";
-        print STDERR "-- ERROR:  This could be caused by an incorrect genomeSize or poor quality\n";
-        print STDERR "-- ERROR:  reads that cound not be sufficiently corrected.\n";
-        print STDERR "-- ERROR:\n";
-        print STDERR "-- ERROR:  You can force Canu to continue by decreasing parameters\n";
-        print STDERR "-- ERROR:  minInputCoverage and/or stopOnLowCoverage.  Be warned that the\n";
-        print STDERR "-- ERROR:  quality of corrected reads and/or contiguity of contigs will be poor.\n";
-        print STDERR "-- \n";
-
-        caExit("", undef);
-    }
-
-  finishStage:
+    addToReport("${tag}SeqStore", $hist);
     generateReport($asm);
 
-    #  DO NOT resetIteration() here.  This function runs to completion on every
-    #  restart, so resetting would obliterate the iteration count.
-    #resetIteration("$tag-sqStoreCreate");
-
-  allDone:
-    stopAfter("sequenceStore");
-
-    return(1);
+    #return($hist);
 }
+
+
+
+#  
+#sub updateSequenceStore ($$) {
+#    my $asm    = shift @_;
+#    my $tag    = shift @_;
+#    my $bin    = getBinDirectory();
+#
+#    if (! -e "$asm.seqStore/readlengths-$tag.txt") {
+#        addToReport("${tag}SeqStore", generateReadLengthHistogram($tag, $asm));
+#        stashSeqStore($asm);
+#    }
+#    generateReport($asm);
+#}
